/******************************************************************
 * appMain.c++                                                    *
 ******************************************************************/
#include <stdlib.h>
#include <strings.h>

#include <Performer/pf/pfChannel.h>
#include <Performer/pf/pfGroup.h>
#include <Performer/pf.h>
#include <Performer/pf/pfDCS.h>
#include <Performer/pfdu.h>
#include <pfcave.h>

#include "wandClientC.h"
#include "myPfWand.h"
#include "PSClass.h"
#include "cncreteMed.h"
#include "happySphere.h"
#include "caveNav.h"

int
main (int argc, char **argv)
{

	// PSClass creates a performer scene and does pfCAVE initialization.
	PSClass * psclassObj = new PSClass(argc, argv);
	psclassObj->configScene();

	pfScene * scene = psclassObj->getScene();
	// group is already parented to our current scene
	pfGroup *grp = psclassObj->getGroup();

	pfDCS * navDCS = new pfDCS;
	grp->addChild(navDCS);

	// instantiate wand container class
	wandClientC * wandCC = new wandClientC;
	wandCC->init(grp);

	// add our wand DCS to pfScene
	navDCS->addChild(wandCC->getSelf()->getDCS());

	// create happyshpere and child it to root scene node.
	happySphere * sphere = happySphere::instance();
	navDCS->addChild(sphere->getDCS());

	// create our GUI concrete mediator
	cncreteMed * mediator = new cncreteMed(scene);

	// simple cave nav class
	caveNav * cNav = new caveNav(navDCS);
	cNav->setInitTranslation(0.0, -2.5, -5.0);

	int wOver;
	while (!CAVEgetbutton(CAVE_ESCKEY))
	{
		wandCC->process();
		// performer graphics
		pfSync();
		pfCAVEPreFrame();
		pfFrame();
		pfCAVEPostFrame();

		// GUI. handleMenu() returns false if no widgets are intersected
		wOver = mediator->handleMenu();

		if (!wOver) // only navigate if we're not fiddling with widgets.
			cNav->navigate();
	}

	pfCAVEHalt();
	fprintf(stderr, "bye.\n");
	pfExit();

	return 0;
}

