/*************************************************************************
 **                           caveNav.h                                 **
 *************************************************************************
 ** caveNav is a simple CAVE navigation class.                          **
 ** Author: Fred Dech                                                   **
 ** (C) 2000 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/

#ifndef _caveNav_h_
#define _caveNav_h_

#include <Performer/pf.h>
#include <Performer/pf/pfDCS.h>
#include <pfcave.h>
#include <sys/types.h>

class caveNav
{
public:
	// constructor
	caveNav(pfDCS * dcs) : _navDCS(dcs), _tSpeed(0.1f),
		_rSpeed(2.5f), _jPad(0.1f), _first(True), _iDx(0.0f),
		_iDy(0.0f), _iDz(0.0f), _jx(0.0), _jy(0.0)
	{ }

	// destructor
	virtual ~caveNav() { }

	void setInitTranslation(float x, float y, float z)
	{
		_iDx = x;
		_iDy = y;
		_iDz = z;
	}

	void setTransSpeed(float ts)     { _tSpeed = ts; }
	void setRotSpeed(float rs)       { _rSpeed = rs; }
	void setJoystickPad(float jpad)  { _jPad = jpad; }

	void navigate(void);

private:
	float _tSpeed, _rSpeed, _jPad;
	float _jx, _jy;
	float _iDx, _iDy, _iDz;
	pfDCS * _navDCS;
	float _wv[3];
	int _first;
};

#endif

