/*************************************************************************
 **                         cncreteMed.h                                **
 *************************************************************************
 ** concrete representation of the abstract vwMediator class.           **
 ** load and manages GUI objects and their signalling to Observer       **
 ** object types.                                                       **
 ** this mediator operates under the model that there is one Main       **
 ** Menu which stays active always.  Main Menu widgets may spawn up     **
 ** to one Sub Menu per widget.  When a Sub Menu is created, the Main   **
 ** Menu still remains active.  In this example mediator, only one      **
 ** widget in the Main Menu will spawn a Sub Menu.  the Sub Menu will   **
 ** be a generic Slider with a Value Button that displays the slider's  **
 ** value dynamically.                                                  **
 ** Author: Fred Dech                                                   **
 ** (C) 2000 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/

#ifndef _cncreteMed_h_
#define _cncreteMed_h_

#include <Performer/pf/pfScene.h> 
#include <Performer/pf/pfSCS.h>
#include <Performer/pf/pfDCS.h>
#include <vwMediator.h>
#include <vwButton.h>

#include "myEnums.h"
#include "childMgr.h"
#include "myObserv.h"

const int  MAX_ELEMENTS = 16; // number of widgets a menu may have
const int  MAX_CHILDREN = 1; // number of sub-menus a widget may have
const int MAX_INTERFACES = 2; // max seperate interfaces supported here

const char _cncreteMed_Config_Path[64] = "./etc/gui_1.config";

// Concrete Mediator for this set of Observer Pattern user interfaces.
class cncreteMed : public vwMediator
{
public:
	// constructor
	cncreteMed(pfScene *, const char * m=_cncreteMed_Config_Path);

	// destructor
	~cncreteMed();

	// run in default Main Menu mode
	int handleMenu();

    inline mainInterfaceState getState()        { return _state; }
    inline void setState(mainInterfaceState s)  { _state = s; }

private:
	childMgr * _childMgr;
	int _nuChildIndex;

	// this function is pure virtual in abstract vwMediator class.
	int createWidgets(char *);

	int createMainInterface(vwWidget::wdgStruct *, int);
	int createSliderInterface(vwWidget::wdgStruct *, int);

	pfScene * _scene;
	pfFont  * _font;
	pfGroup * _ifaceGroup; // grouping node all interfaces are childed to

	myObserv * _myObserv;  // pointer to our external interface observer

	// mediator state variable. defined in myEnums.h
	mainInterfaceState _state;

	// mediator sub-state variable. defined in myEnums.h
	mainInterfaceType   _interfaceType;

	vwWidget::wType _wtype;

	int _wCount;  // total widget count

	pfDCS * _iDCS[MAX_INTERFACES];  // each interface dcs goes in list
	int _lastiDCS;  // holds count of total number of interfaces open

	pfLightSource * _lite; // light source for main menu.

	// pointer list of all widgets created in this concrete mediator
	vwWidget * _wdgList[MAX_ELEMENTS * MAX_INTERFACES];

	// used in interface setup routine.
	// pass it a string and get back corresponding mainButton::mbFunction
	buttonIdentType extractBIdent(char *);

	// handle button with pressed condition
	void handleButton(vwButton *, int);

	void cleanUp(int);  // clean child out of scene-graph
};

#endif

