/*************************************************************************
 **                          happySphere.h                              **
 *************************************************************************
 ** sphere class for illustration purposes.                             **
 ** Author: Fred Dech                                                   **
 ** (C) 2001 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/

#ifndef _happySphere_h_
#define _happySphere_h_

#include <Performer/pf/pfDCS.h>
#include <Performer/pfdu.h>
#include <Performer/pf.h>

class happySphere
{
protected:
	// constructor  we only need one of these happy shpheres.
	happySphere();

public:
	// destructor
	~happySphere()
	{
		if (_dcs2 && _dcs1) {
			_dcs1->removeChild(_dcs2);
			pfDelete(_dcs1);
			pfDelete(_dcs2);
		}
	}

	// singleton method
	static happySphere * instance()
	{
		if (_instance == NULL) {
			_instance =  new happySphere();
		}
		return _instance;
	}

	inline pfDCS * getDCS() { return _dcs1; }
	inline void setScale(float scale)
	{
		_dcs1->setScale(scale, scale, scale);
	}

	inline void setWidth(float width)
	{
		static float iwdth = width;

		_dcs2->setScale(iwdth, iwdth, width);
	}

private:
	static happySphere * _instance;

	pfDCS * _dcs1;
	pfDCS * _dcs2;

};

#endif

