/**************************************************************************
 **                              myEnums.h                               **
 **************************************************************************
 ** Enumerated types for Widgets used in this application                **
 ** NOTE:  it is likely that not all State and Indent enums are used in  **
 ** this program.  it is also likely that some Scope classes aren't even **
 ** used.                                                                **
 **************************************************************************/

#ifndef _myEnums_h_
#define _myEnums_h_

#ifdef _STANDARD_C_PLUS_PLUS
    // If -LANG:std is specified, it defines the macro
    // in the preceding line.  Use new-style headers
    // and a using directive to bring names from the
    // std namespace into the global namespace.
#include<complex>
#include<iostream>
using namespace std;

#else
    // If -LANG:std is not specified, use old-style headers,
    // and there is no need for a using directive.
#include<complex.h>
#include<iostream.h>

#endif

#include <hash_map>

class mainInterfaceStateScope
{
public:
	enum mainInterfaceState {
		PARENT   = 0,
		SCALE    = 2,
		OBSERVER = 3,
		NA       = -1
	};
};

typedef mainInterfaceStateScope::mainInterfaceState mainInterfaceState;

//-----------------------------------------------------------------------

class mainInterfaceTypeScope
{
public:
	enum mainInterfaceType {
		MAIN_MENU = 0,
		SLIDER    = 1,
		NA        = -1
	};
};

typedef mainInterfaceTypeScope::mainInterfaceType mainInterfaceType;

//-----------------------------------------------------------------------

class buttonInterfaceTypeScope
{
public:
	enum buttonInterfaceType {
		MAIN      = 0,
		SLIDER    = 1,
		NA        = -1
	};
};

typedef buttonInterfaceTypeScope::buttonInterfaceType buttonInterfaceType;

//-----------------------------------------------------------------------

class buttonIdentTypeScope
{
public:
	enum buttonIdentType {
		SLIDER       = 10,
		MANIP        = 0,
		SCALE        = 1,
		UNKNOWN      = 2,
		FANCY        = 4,
		MAG1         = 7,
		MAG2         = 8,
		RESET        = 9,
		DOANOTHER    = 11,
		DOATHIRD     = 12,
		NA           = -1
	};

	// hash_map key equality function
	struct eqstr {
		bool operator()(const char* s1, const char* s2) const
		{
			// returns TRUE if there's a match, else FALSE
			return strcmp(s1, s2) == 0;
		}
	};


	// shorten this down a bit for here and cncreteMed
	typedef
	hash_map<const char *, buttonIdentType, hash<const char *>, eqstr> myMap;

	static myMap btnMap;

	// fills our hashed associative array for ID'ing buttons
	// called from cncreteMed constructor.
	static void fillBtnMap() {
		btnMap ["SLIDER"] = buttonIdentTypeScope::SLIDER;
		btnMap ["MANIP"] = buttonIdentTypeScope::MANIP;
		btnMap ["SCALE"] = buttonIdentTypeScope::SCALE;
		btnMap ["FANCY"] = buttonIdentTypeScope::FANCY;
		btnMap ["UNKNOWN"] = buttonIdentTypeScope::UNKNOWN;
		btnMap ["MAG1"] = buttonIdentTypeScope::MAG1;
		btnMap ["MAG2"] = buttonIdentTypeScope::MAG2;
		btnMap ["RESET"] = buttonIdentTypeScope::RESET;
		btnMap ["NA"] = buttonIdentTypeScope::NA;
		btnMap ["DOANOTHER"] = buttonIdentTypeScope::DOANOTHER;
		btnMap ["DOATHIRD"] = buttonIdentTypeScope::DOATHIRD;
	}
};

typedef buttonIdentTypeScope::buttonIdentType buttonIdentType;

//-----------------------------------------------------------------------

class sliderInterfaceTypeScope
{
public:
	enum sliderInterfaceType {
		SLIDER      = 0,
		NA          = -1
	};
};

typedef sliderInterfaceTypeScope::sliderInterfaceType sliderInterfaceType;

//-----------------------------------------------------------------------

class sliderIdentTypeScope
{
public:
	enum sliderIdentType {
		SLIDER1    = 8,
		NA         = -1
	};
};

typedef sliderIdentTypeScope::sliderIdentType sliderIdentType;

//-----------------------------------------------------------------------


#endif // #ifndef _myEnums_h_

