/*************************************************************************
 **                          myObserv.c++                               **
 *************************************************************************
 ** Observer interface between GUI and application.                     **
 ** Author: Fred Dech                                                   **
 ** (C) 2000 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/
#include <pfcave.h>

#include "myObserv.h"
#include <vwSlider.h>

/////////// myObserv Singleton instance ////////////////////
myObserv * myObserv::_instance = NULL;

myObserv::myObserv(vwMediator *m)
	: vwObserver(m), _yRot(0.0), _dcs(NULL)
{

	if (!m) {
		fprintf(stderr, "myObserv::myObserv: Construction ");
		fprintf(stderr, "with NULL vwMediator pointer is not allowed.\n");
		exit(1);
	}
	// get an instance of the happySphere, which is also a singleton
	_sphere = happySphere::instance();
	// get the happyShphere's pfDCS.
	_dcs = _sphere->getDCS();
}

// singleton method
myObserv *
myObserv::instance(vwMediator * m)
{
	if (_instance == NULL) {
		_instance = new myObserv(m);
	}
	return _instance;
}

// override of pure virtual in abstract vwObserver.
// here's where the slider modifies the sphere's pfDCS.
void
myObserv::update(vwSubject *s, void *data)
{
	// we expect either a slider or the RESET button
	vwSlider * slider = dynamic_cast<vwSlider *>(s);
	vwButton * button = dynamic_cast<vwButton *>(s);

	if (slider) {       // slider
		float sVal = *(static_cast<float *>(data));
		_yRot = sVal;

	} else if (button)  // RESET
		_yRot = 0.0;
	else                // something we didn't expect.  blow.
		return;

		if (_dcs)
			_dcs->setRot(_yRot, 0.0, 0.0);

}

