/*************************************************************************
 **                         alphaObserv.h                               **
 *************************************************************************
 ** This concrete Observer handles events from the sliders and the      **
 ** RESET button.                                                       **
 ** It simply allows one to modify the transparency of the single       **
 ** geometry associated with its slider.  To make it work with a slider **
 ** that effects all the children under it, consider doing something    **
 ** with the associative STL map< > template, or a reference list with  **
 ** parent and number of children and pointers to those children,       **
 ** initial alpha and current alpha.                                    **
 *************************************************************************/

#ifndef _alphaObserv_h_
#define _alphaObserv_h_

#include <Performer/pf.h>
#include <vwMediator.h>
#include <dependents.h>

#include "myDefs.h"
#include <Performer/pf/pfDCS.h>

const char _ALPHA_OBSERV_MODEL_PATH[64] = "./models/";

class alphaObserv : public vwObserver
{
protected:
	alphaObserv(vwMediator *, pfDCS *);

public:
	virtual ~alphaObserv() { }

	static alphaObserv * instance(vwMediator *m, pfDCS *gdcs)
	{
		if (_instance == NULL) {
			_instance = new alphaObserv(m, gdcs);
		}

		return _instance;
	}
	// allow non-mediator classes access to alphaObserv
	// note: if it hasn't been instatiated, it will return NULL, so they
	// need to check the return pointer's value.
	static alphaObserv * instance(void)
	{
		return _instance;
	}

	inline float getCurrentAlpha(buttonIdentType type)
	{
		return alphaList[type].currentAlpha;
	}

	// resets pfTransparency to PFTR_FAST for geostate of type and
	// any children.
	inline void pftrFast(buttonIdentType type)
	{
		_updateEvent = False;
		alphaMod(&(alphaList[type]));
	}

	// override pure virtual from abstract vwObserver class
	void update(vwSubject *, void *);

private:
	static alphaObserv * _instance;

	// load the geometries who's alpha values we'll manipulate
	int loadModel();

	void reset();
	void changeTransp(pfNode *, float);

	// this lets alphaMod switch between PFTR_HIGH_QUALITY and PFTR_FAST.
	int _updateEvent;

	pfDCS * _geomdcs;

	void init();

	// our struct
	typedef struct {
		buttonIdentType ident;  // parent button ID
		float currentAlpha;     // alpha value of the geom
		int attached;           // flag telling if attached to scenegraph
		pfGroup * geom;         // pfGroup of geode represented here
	} alphaStrct;

	// our alpha array
	alphaStrct alphaList[buttonIdentMax];

	// modify alpha value(s)
	int alphaMod(alphaStrct *);

};

#endif

