// childMgr.c++
#include "childMgr.h"

////// Child Manager Singleton Class ///////
childMgr * childMgr::_instance = NULL;

// constructor
childMgr::childMgr() : _cStruct(NULL), _childDepth(0)
{ }

// destructor
childMgr::~childMgr()
{
	clear();
}

// instance of singleton
childMgr *
childMgr::instance()
{
	if (_instance == NULL) {
		_instance = new childMgr;
	}

	return _instance; 
}

int
childMgr::addChild(int index, vwWidget * parent, buttonIdentType itype)
{
	int cnt = 0;
	chldStrct * strct = _cStruct;

	if (_childDepth == 0) {
		_cStruct = new chldStrct;
		strct = _cStruct;
	}
	else {
		cnt++;
		while (cnt < _childDepth) {
			strct = strct->next;
			cnt++;
		}
		strct->next = new chldStrct;
		strct = strct->next;
	}
	_childDepth++;

	strct->parentIndex = index;
	strct->parent = parent;
	strct->iType = itype;

	// initialize to sane values
	strct->next = NULL;
	strct->index = -1;
	strct->count = 0;

	return cnt;  // return index into linked list of new struct
}

int
childMgr::getChildStartIndex(int childNum)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return False;

	return strct->index;
}

void
childMgr::setChildStartIndex(int childNum, int index)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return;

	strct->index = index;
}

int
childMgr::getLastUsedIndex()
{
	chldStrct * strct = getStrct(_childDepth - 1);

	if (!strct)
		return False;

	return strct->index + strct->count - 1;
}

int
childMgr::getNumWidgets(int childNum)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return False;

	return strct->count;
}

void
childMgr::setNumWidgets(int childNum, int count)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return;

	strct->count = count;
}

vwWidget *
childMgr::getParentWidget(int childNum)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return NULL;

	return strct->parent;
}

int
childMgr::getParentIndex(int childNum)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return -1;

	return strct->parentIndex;
}

buttonIdentType
childMgr::getParentIdent(int childNum)
{
	chldStrct * strct = getStrct(childNum);

	if (!strct)
		return buttonIdentTypeScope::NA;

	return strct->iType;
}

int
childMgr::getTotalChildWidgets()
{
	if (_childDepth == 0)
		return False;

	int cnt = 0;
	int total = 0;

	chldStrct * strct = _cStruct;

	while (cnt < _childDepth-1) {
		total += strct->count;
		strct = strct->next;
		cnt++;
	}

	return total;
}


// remove child at end of list
int
childMgr::removeChild()
{
	int cnt = 0;
	chldStrct * strct = _cStruct;
	chldStrct * ostrct = NULL;

	if (!strct) return False;

	while (cnt < _childDepth - 1) {
		ostrct = strct;
		strct = _cStruct->next;
		cnt++;
	}
	delete strct;

	if (ostrct)
		ostrct->next = NULL;
	else
		_cStruct = NULL;

	_childDepth--;

	return True;
}

// clear out entire list of children
void
childMgr::clear()
{
	chldStrct * start = _cStruct;
	chldStrct * prev = NULL;

	while (start != NULL) {
		_cStruct = start;
		prev = NULL;
		while (_cStruct->next != NULL) {
			prev = _cStruct;
			_cStruct = _cStruct->next;
		}
		delete _cStruct;

		if (prev)
			prev->next = NULL;
		else
			start = NULL;
	}
	_childDepth = 0;
}

