/************************************************************************
 **                         childMgr.h                                 **
 ************************************************************************
 ** widget child manager.  maintains a list of all current children    **
 ** and where they came from (who their parents are).  this class is   **
 ** necessary for proper GUI management where multiple child layers    **
 ** are possible.                                                      **
 ** Author: Fred Dech                                                  **
 ** (C) 2001 Virtual Reality in Medicine Laboratory                    **
 **          University of Illinois at Chicago                         **
 ************************************************************************/

#ifndef _childMgr_h_
#define _childMgr_h_

#include <vwWidget.h>
#include "myDefs.h"

// singleton class for managing child interfaces.
class childMgr
{
public:
	// singleton instantiation method
	static childMgr * instance();

	// destructor
	~childMgr();

	inline int getChildDepth()  // return number of child interfaces open
	{
		return _childDepth;
	}
	void setChildStartIndex(int, int); // set index where spec'd child starts
	int getChildStartIndex(int);   // return index where spec'd child starts
	int getLastUsedIndex();        // return last widget index
	void setNumWidgets(int, int);  // set number of widgets in spec'd child
	int getNumWidgets(int);        // return number of widget in spec'd child
	int getTotalChildWidgets();    // return total num of child widgets
	int getParentIndex(int);       // get index of parent of spec'd child
	vwWidget * getParentWidget(int); // return pointer to parent widget

	buttonIdentType getParentIdent(int);  // get spec'd child's parent
                                         // buttonIdentType

	// returns the index for this new child data struct
	int addChild(int index, vwWidget * parent, buttonIdentType bit);

	int removeChild();   // removes the last child from the list
	void clear();        // remove all children from the list

protected:
	// constructor
	childMgr();

private:
	static childMgr * _instance;

	int _childDepth;             // depth of child levels currently open

	typedef struct chld {
		int index;               // start index of this child
		int parentIndex;         // index of parent of child
		vwWidget * parent;       // pointer to parent of child
		int count;               // number of widgets in this child
		buttonIdentType iType;   // buttonIdentType of Parent
		chld * next;             // next link
	} chldStrct;

	chldStrct * _cStruct;

	// this code gets called by almost every method
	// so we'll try and inline it.
	inline chldStrct * getStrct(int childNum)
	{
		if (_childDepth <= childNum)
			return NULL;

		int cnt = 0;
		chldStrct * strct = _cStruct;

		while (cnt < childNum) {
			strct = strct->next;
			cnt++;
		}

		return strct;
	}
};


#endif

