/*************************************************************************
 **                          wandClientC.c++                           **
 *************************************************************************
 ** simple wand container class                                         **
 *************************************************************************/
#include "wandClientC.h"

#include <Performer/pf/pfDCS.h>
#include <Performer/pf/pfTraverser.h>
#include <Performer/pfdu.h>
#include <pfcave.h>
#include <Performer/pf/pfGroup.h>
#include <Performer/pf/pfGeode.h>
#include <Performer/pr/pfGeoSet.h>
#include <Performer/pr/pfMaterial.h>
#include <Performer/pr/pfGeoState.h>

wandClientC::~wandClientC()
{
	while (_wandCount > 0)
		delete w[--_wandCount];
}

int
wandClientC::init(pfGroup *grp)
{
	_wccGroup = grp;
	myPfWand * w;
	w = new myPfWand();
	// put the wand avatar into our dbase
	add(w);

	return 1;
}

void
wandClientC::process(int iter)
{
	// update our wand pfDCS from local tracker
	updateSelf();
}

void
wandClientC::add(myPfWand * wand)
{
	pfDCS * dcs;

	// if its other than our Wand (1st Wand) being added
	if (_wandCount > 0 && _wandCount < _WAND_CONT_MAX_CLIENTS) {
		dcs = wand->getDCS();
		_wccGroup->addChild(dcs);
	} else if (_wandCount == 0) {
		fprintf(stderr, "ADDING OUR WAND\n");
		dcs = wand->getDCS();
		// set another seperate pointer to local wand
		_selfWand = wand;
	}
	fprintf(stderr, "wandClientC::add():  wandCount=%d\n", _wandCount+1);

	// change color of wand to something other than default (local wand).
	changeWandColor(dcs, wandAmbient[_wandCount], wandDiffuse[_wandCount]);

	w[_wandCount++] = wand;

}

int
wandClientC::remove(int i)
{
	if (i >= 0 && i < _wandCount) {
		delete w[i];

		for (int nn=i+1; nn < _wandCount; nn++,i++)
			w[i] = w[nn];

		w[nn] = NULL;
		_wandCount--;

		fprintf(stderr, "wandClientC::remove():  wandCount=%d\n",
		                 _wandCount+1);
		return 1;
	} else
		return 0;
}

int
wandClientC::remove(long id)
{
	int ii;

	for (ii=0; ii< _wandCount; ii++) {
		if (id == w[ii]->getID()) {
			_wccGroup->removeChild(w[ii]->getDCS());
			delete w[ii];
			w[ii] = NULL;
			break;
		}
	}

	if (ii < _wandCount && w[ii] == NULL) {
		for (int nn=ii+1; nn < _wandCount; nn++, ii++) {
			w[ii] = w[nn];
			w[nn] = NULL;
		}
		_wandCount--;
	}
	fprintf(stderr, "wandClientC::remove():  wandCount=%d\n", _wandCount+1);

	return _wandCount;
}

void
wandClientC::updateSelf()
{
	float wandP[3], wandO[3], wandV[3];

	pfDCS * dcs = _selfWand->getDCS();

	// get items of interest
	pfCAVEGetPosition(CAVE_WAND_NAV, wandP);
	pfCAVEGetOrientation(CAVE_WAND_NAV,wandO);
	CAVEGetVector(CAVE_WAND_FRONT_NAV, wandV);

	// update our wand's coords
	memcpy((void *)(_selfWand->pos), (void *)wandP, 3 * sizeof(float));
	memcpy((void *)(_selfWand->ori), (void *)wandO, 3 * sizeof(float));
	memcpy((void *)(_selfWand->vec), (void *)wandV, 3 * sizeof(float));

	dcs->setTrans(wandP[0], wandP[1], wandP[2]);
	dcs->setRot(wandO[2],wandO[0],wandO[1]);

}

int
wandClientC::changeWandColor(pfNode *node, float * acolor, float * dcolor)
{
	pfMaterial * mtl =
	      (pfMaterial *)pfMalloc(sizeof(pfMaterial), pfGetSharedArena());

	mtl->setSide(PFMTL_FRONT);

	mtl->setColor(PFMTL_AMBIENT, *acolor, *(acolor+1), *(acolor+2));
	mtl->setColor(PFMTL_DIFFUSE, *dcolor, *(dcolor+1), *(dcolor+2));
	mtl->setColor(PFMTL_SPECULAR, *dcolor, *(dcolor+1), *(dcolor+2));
	fprintf(stderr, "wandClientC::changeWandColor(): dif: %f %f %f\n",
	                 *dcolor, *(dcolor+1), *(dcolor+2));
	fprintf(stderr, "wandClientC::changeWandColor(): amb: %f %f %f\n",
	                 *acolor, *(acolor+1), *(acolor+2));

	mtl->setShininess(35.0);  // 0.0 to 128.0
	mtl->setColorMode(PFMTL_FRONT, PFMTL_CMODE_OFF);
	pfGroup *group = (pfGroup *)node;
	pfGeode *geode = dynamic_cast<pfGeode *>(group->getChild(0));

	pfGeoSet *gset = geode->getGSet(0);

	pfGeoState *gstate = new pfGeoState;

	// configure gstate.
	gstate->setAttr(PFSTATE_FRONTMTL, mtl);
	gstate->setAttr(PFSTATE_BACKMTL, mtl);

	gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
	gset->setGState(gstate);
	geode->insertGSet(0,gset);

	return 1;
}

