/*************************************************************************
 **                            PSClass.c++                              **
 *************************************************************************
 ** a simple C++ container for the lower portion of the Performer       **
 ** scenegraph & some attributes.                                       **
 *************************************************************************/
#include "PSClass.h"
#include <pfcave.h>

PSClass::PSClass(int argc, char **argv)
{
	// set far CAVEclipping plane
	CAVEFar = 1000;

	// initialize performer
	pfInit();

	// Equivalent to CAVE Config. Must be called after pfInit()
	pfCAVEConfig(&argc,argv,NULL);

	// load iv, pfb dsos for shared memory use
    // should be called before pfConfig().
    pfdInitConverter("iv");
    pfdInitConverter("pfb");

    // Performer config
    pfConfig();

    // Creates all the channels for performer
    pfCAVEInitChannels();

    // Create a scene
    _scene = new pfScene;
}

// put in a few lights & stuff
// amb_color is a simple float of 6 values amb[3], color[3] per light
// position is a float position vector xyzw per light
int
PSClass::configScene(int numLights, float * pos, float * clr)
{
	int createFlag = False;

	// default
	if (pos == NULL && clr == NULL) {
		numLights = 2;
		createFlag = True;
		pos = new float[6];
		clr = new float[12];
		float *p = pos;
		float *c = clr;

		*p = -1.0; *(p+1) = -1.0; *(p+2) = -1.0; p+=3;
		*p = 1.0; *(p+1) = -0.2; *(p+2) = -0.2;
		*c = 0.5; *(c+1) = 0.5; *(c+2) = 0.5; c+=3;
		*c = 0.75; *(c+1) = 0.75; *(c+2) = 0.75; c+=3;
		*c = 0.5; *(c+1) = 0.5; *(c+2) = 0.5; c+=3;
		*c = 1.0; *(c+1) = 1.0; *(c+2) = 1.0;
	}
	if (pos && clr) {
		pfGeoState *gstate1 = new pfGeoState;
		_scene->setGState(gstate1);
		float * p = pos; float * c = clr;
		if (numLights > _PSC_MAX_LIGHTS)
			numLights = 5;

		for (int nn=0; nn<numLights; nn++, p+=3) {
			lt[nn] = new pfLightSource;
			lt[nn]->setAmbient(*c,*(c+1),*(c+2)); c+=3;
			lt[nn]->setColor(PFLT_DIFFUSE, *c, *(c+1),*(c+2)); c+=3;
			lt[nn]->setPos(*p, *(p+1), *(p+2), 0.0f);

			_scene->addChild(lt[nn]);
		}
	}
	if (createFlag) {
		delete [] pos;
		delete [] clr;
	}

	_grp = new pfGroup;

	_grp->setTravMask(PFTRAV_ISECT,
	                  0x8,
	                  PFTRAV_SELF | PFTRAV_DESCEND |
	                  PFTRAV_IS_CACHE, PF_SET);

	_scene->addChild(_grp);

	pfChannel *chan = pfCAVEMasterChan();
	chan->setTravMode(PFTRAV_MULTIPASS, PFCULL_SORT);

	// make sure transparency is done as planned
	static int sort[] = { PFSORT_BACK_TO_FRONT };
	chan->setBinSort(PFSORT_TRANSP_BIN, PFSORT_TRANSP_BIN_ORDER, sort);

	chan->setScene(_scene);


	return 1;
}

