/*************************************************************************
 **                         fancyButton.c++                             **
 *************************************************************************
 ** fancyButton subclass of vwButton.                                   **
 ** this button magnifies and minifies the slider widgets to which it   **
 ** is attached.  it must keep track of both states, and must be        **
 ** attached to all the widgets in its local interface in order to have **
 ** the most up-to-date state data.                                     **
 ** Author: Fred Dech                                                   **
 ** (C) 2000 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/
#include "fancyButton.h"
#include "fancyInc.h"

// modify fancyButton (Magnify) database when other widgets change
// value
void
fancyButton::update(vwSubject *s, void *data)
{
	float val = *(static_cast<float *>(data));

	vwSlider * slider = dynamic_cast<vwSlider *>(s);
	fancyInc * inc = dynamic_cast<fancyInc *>(s);

	if (slider) {
		if (slider == _cSlider) {
			_fancyParms.center = val;
		} else {
			_fancyParms.width = val;
		}

	 } else if (inc) {
		if (inc->getIdent() == incrementorIdentTypeScope::WIDTH1) {
			_fancyParms.width = inc->getWidth();
		} else {
			_fancyParms.center = inc->getCenter();
		}
	} else
		return;


	// make working MAG range twice the width
	float range = _fancyParms.width * 2.0;
	float rangelo = _fancyParms.center - range * 0.5;
	float rangehi = _fancyParms.center + range * 0.5;

	if (_fancyParms.width >= 0) { 
		_fancyParms.rangeLo[1] = rangelo > _fancyParms.rangeLo[0] ?
			rangelo : _fancyParms.rangeLo[0];

		_fancyParms.rangeHi[1] = rangehi < _fancyParms.rangeHi[0] ?
			rangehi : _fancyParms.rangeHi[0];
	}
}

// override hook into vwButton::setPressed()
// center slider needs to be modified FIRST before notify(), it then get's
// updated and thus serves as the parameter basis for the other widget updates.
// remember vwButton::setPressed() calls hook doProcessPress() before it
// callls notify().
void
fancyButton::doProcessPress()
{
	if (getPressed()) {
		// re-init center slider to MAG values
		_cSlider->setDataParms(_fancyParms.rangeLo[1],
		                       _fancyParms.rangeHi[1],
		                       _fancyParms.center);
	} else {
		// re-init center slider to ORIGINAL values
		_cSlider->setDataParms(_fancyParms.rangeLo[0],
		                       _fancyParms.rangeHi[0],
		                       _fancyParms.center);
	}

}

