/*************************************************************************
 **                         fancyButton.h                               **
 *************************************************************************
 ** fancyButton subclass of vwButton.                                   **
 ** this button magnifies and minifies the slider widgets to which it   **
 ** is attached.  it must keep track both states, and must be attached  **
 ** to all the widgets in its local interface in order to have the      **
 ** most up-to-date state data.                                         **
 ** Author: Fred Dech                                                   **
 ** (C) 2000 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/

#ifndef _fancyButton_h_
#define _fancyButton_h_

#include <vwButton.h>
#include <vwSlider.h>

#include "myEnums.h"

class fancyButton : public vwButton, public vwObserver
{
public:
	// constructor.
	fancyButton(vwMediator *m, pfDCS * dcs) :
	     vwButton(m,dcs), vwObserver(m),
		_bIface(buttonInterfaceTypeScope::FANCY)
	{ }

	virtual ~fancyButton() { }

	// override pure virtual vwObserver::update()
	void update(vwSubject *, void *);

	// declare/define set's for vwWidget::getIdent() & getInterface()
	inline void setInterface(buttonInterfaceType f)  { _bIface = f; }
	inline void setIdent(buttonIdentType f)          { _bIdent = f; }

	// override vwWidget funcs
	int getInterface()          { return _bIface; }
	int getIdent()              { return _bIdent; }

	// set/get a pointer to a slider [should be center slider]
	inline void setCenterSlider(vwSlider *s)   { _cSlider = s; }
	inline vwSlider * getCenterSlider()        { return _cSlider; }

	// external data interfaces
	inline float getCenter()  { return _fancyParms.center; }
	inline float getWidth()   { return _fancyParms.width; }

	inline void getOrigScaleRange(float *lo, float *hi) {
		*lo = _fancyParms.rangeLo[0];
		*hi = _fancyParms.rangeHi[0];
	}
	inline void getMagScaleRange(float *lo, float *hi) {
		*lo = _fancyParms.rangeLo[1];
		*hi = _fancyParms.rangeHi[1];
	}

	inline void setCenter(float v)  { _fancyParms.center = v; }
	inline void setWidth(float v)   { _fancyParms.width = v; }

	inline void setOrigScaleRange(float lo, float hi) {
		_fancyParms.rangeLo[0] = lo;
		_fancyParms.rangeHi[0] = hi;
	}
	inline void setMagScaleRange(float lo, float hi) {
		_fancyParms.rangeLo[1] = lo;
		_fancyParms.rangeHi[1] = hi;
	}

	// override vwButton::doProcessPress() hook into press()
	void doProcessPress();

private:
	// maintain standard and magnified fancyment data params.
	struct {
		float width;   // data width
		float center;  // data center
		float rangeLo[2]; // data range lo
		float rangeHi[2]; // data range high
	} _fancyParms;

	vwSlider * _cSlider; // MAG re-inits and updates center slider which
						 // then notifies the other widgets.

	buttonIdentType    _bIdent;     // button ID
	buttonInterfaceType _bIface;    // button interface Type
};

#endif

