/*************************************************************************
 **                          fancyInc.c++                               **
 *************************************************************************
 ** this is a special-purpose vwIncrementor designed to be used         **
 ** exclusively in a fancy slider interface.                            **
 ** Author: Fred Dech                                                   **
 ** (C) 1999 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/
#include <Performer/pfdu.h>

#include "fancyInc.h"
#include "fancyButton.h"

fancyInc::fancyInc(vwMediator *m, pfSCS *scs, pfFont *f) :
	vwIncrementor(m,scs,f), vwObserver(m), _width(-1.0),
	_center(-1.0), _geomMin(0.0), _geomMax(0.0),
	_iIface(incrementorInterfaceTypeScope::FANCY)
{
	_dataMin = 0.0;
}

// override of vwIncrementor::setValue()
void
fancyInc::setValue(float dval)
{
	char txt[256];
	_value = dval;

	if (_dataRange != 0.0) {
		if (_iIdent == incrementorIdentTypeScope::WIDTH1) {
			_width = _value;
			_geomWidth = (_width * _geomRange) / _dataRange;
		} else {
			_center = _value;
			_geomCenter = _geomMin +
	            ((_center - _dataMin) * _geomRange) / _dataRange;
		}
	}
	// modify 2nd line of label with new data value.
	if (_label) {
		_label->pftxt->removeString(_label->pfstr[1]);
		if (_iType == INTEGER)
			sprintf(txt, "%d", (int)dval);
		else
			sprintf(txt, "%.4f", dval);

		_label->pfstr[1]->setMat(_labelMtx);
		sprintf(_label->word_l[1], "%s",txt);
		_label->pfstr[1]->setString(_label->word_l[1]);
		_label->pftxt->addString(_label->pfstr[1]);
	}

}

// update.  normal, 'xept if magnify button, need to change range and inc
// if incrementor.
void
fancyInc::update(vwSubject *s, void *data)
{
	static float inc = _fInc; // vwIncrementor protected inc variable

	vwWidget * wdg = dynamic_cast<vwWidget *>(s);

	if (!wdg->isOfType(vwWidget::wButton)) {
		float v = *(static_cast<float *>(data));

		setValue(v);
	} else {
		// this is a magnify/minify event
		fancyButton * butt = dynamic_cast<fancyButton *>(wdg);
		if (!butt)
			return;

		float lo, hi;

		if (butt->getPressed()) {
			inc = _fInc;
			_fInc = inc / 20.0;  // arbitrary increased precision of _fInc
			butt->getMagScaleRange(&lo, &hi);
		} else {
			_fInc = inc;       // reset _fInc to non-MAG value
			butt->getOrigScaleRange(&lo, &hi);
		}
		if (_iIdent == incrementorIdentTypeScope::WIDTH1) {
			setDataParms(0.0, (hi - lo), _fInc);
			setValue(butt->getWidth());
		} else {
			setDataParms(lo, hi, _fInc);
			setValue(butt->getCenter());
		}
	}
}

float
fancyInc::getGeomVal()
{
	if (_iIdent == incrementorIdentTypeScope::WIDTH1)
		return _geomWidth;
	else
		return _geomCenter;
}

