/*************************************************************************
 **                           fancyObserv.c++                           **
 *************************************************************************
 ** Observer for fancy interface.                                       **
 ** Author: Fred Dech                                                   **
 ** (C) 2000 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/
#include <vwButton.h>
#include "fancyObserv.h"

#include "myEnums.h"

#include "fancyCSlider.h"
#include "fancyWSlider.h"
#include "fancyInc.h"

/////// Fancy Observer Singleton Class ////////
fancyObserv * fancyObserv::_instance = NULL;

// protected constructor
// note: default, arbitrary value settings for parameters.
// in a real application, the go-between observer probably wouldn't set
// any of these.  they would come from the object the observer links to
// the interface.
fancyObserv::fancyObserv(vwMediator *m)
	: vwObserver(m), _level(1.0), _width(1.0), _lo(0.0), _hi(5.0)
{
	if (!m) {
		fprintf(stderr, "fancyObserv::fancyObserv: Construction ");
		fprintf(stderr, "with NULL vwMediator pointer is not allowed.\n");
		exit(1);
	}
	_inc = (_hi - _lo) / 10.0;  // set inc's to 1/10th range (arbitrary).

	// get a instance of happySphere and initialzie it.
	_sphere = happySphere::instance();
	_sphere->setWidth(_width);
	_sphere->setScale(_level);

	_svlev = _level;
	_svwid = _width;
}

fancyObserv *
fancyObserv::instance(vwMediator * m)
{
	if (_instance == NULL) {
		_instance = new fancyObserv(m);
	}
	return _instance;
}

// override of pure virtual in abstract vwObserver.  this function gets
// called each time a vwSubject that this observer is attached to calls
// notify().  notify() is handled internally by widget classes.
void
fancyObserv::update(vwSubject *s, void *data)
{
	float sVal = *((float *)data);

	vwSlider * slider = dynamic_cast<vwSlider *>(s);
	fancyInc * inc = dynamic_cast<fancyInc *>(s);

	if (slider) {
		int ident = slider->getIdent();

		switch (ident) {
		  case sliderIdentTypeScope::SLIDERC1 :
			if (sVal != _level) {
				_level = sVal;
				_sphere->setScale(sVal);
			}
			break;
		  case sliderIdentTypeScope::SLIDERL1 :
		  case sliderIdentTypeScope::SLIDERH1 :
			if (sVal != _width) {
				_width = sVal;
				_sphere->setWidth(sVal);
			}
			break;
		}

	} else if (inc) {
		int ident = inc->getIdent();

		if (ident == incrementorIdentTypeScope::WIDTH1)
		{
			sVal = inc->getWidth();
			if (sVal != _width) {
				_width = sVal;
				_sphere->setWidth(sVal);
			}
		} else {
		    sVal = inc->getCenter();
			if (sVal != _level) {
				_level = sVal;
				_sphere->setScale(sVal);

			}
		}
	}

	else if (vwButton * bt = dynamic_cast<vwButton *>(s)) {
	// it's gotta be the RESET botton
		_sphere->setWidth(_svwid);
		_sphere->setScale(_svlev);
		_level = _svlev;
		_width = _svwid;
	}
}

