/*************************************************************************
 **                           manipulator.c++                           **
 *************************************************************************
 ** CAVE track-ball manipulator for a DCS    *                          **
 ** Author: Fred Dech                                                   **
 ** (C) 1999 Virtual Reality in Medicine Laboratory                     **
 **          University of Illinois at Chicago                          **
 *************************************************************************/
#include "manipulator.h"

void
manipulator::init(pfDCS *dcs)
{
	float wandPos[3];

	_pickedDCS = dcs;

	// grab wands pos vector
	CAVEGetPosition(CAVE_WAND_NAV,wandPos);

	// Grab the transform matrix of the selected object.
	_pickedDCS->getMat(_grabMat);

	// hitLoc has worldcoord values of intersection point on pickedDCS
	// get distance components between picked geom point and wand pos.
	float x, y, z, w;
	_grabMat.getRow(3, &x, &y, &z, &w);
    _dx = wandPos[0] - x;
    _dy = wandPos[1] - y;
    _dz = wandPos[2] - z;

	// remove distance between picked geom centroid and
	// wandPos origin to avoid orbit rotations.
	_grabMat.postTrans(_grabMat, _dx, _dy, _dz);

	pfMatrix invMat;
	getInverseWandMatrix(invMat);

	// matrix mult geom at wand origin matrix with wand inverse matrix
	_grabMat *= invMat;

}

// manipulate picked geom DCS via Wand
pfMatrix *
manipulator::link()
{
	pfMatrix wMat;

	getWandMatrix(wMat);

	// rotate grabbed geom DCS
	// multiply grab_mat by current Wand matrix where grab_mat was computed
	// from grabbed matrix with grabbed origin on wand origin.
	_newMat = _grabMat * wMat;

	// re-translate to correct distance/direction from wand position.
	_newMat.postTrans(_newMat, -_dx, -_dy, -_dz);

	// update object DCS transf matrix.
	_pickedDCS->setMat(_newMat);

	return & _newMat;
}


void
manipulator::listpfMatrix(pfMatrix *m, const char *name)
{
    float c0, c1, c2, c3;

    printf("\nMatrix %s:\n",name);
    for (int i=0; i<4; i++) {
        m->getRow(i, &c0, &c1, &c2, &c3);
        printf("row %d  = %f  %f  %f  %f\n",i,c0,c1,c2,c3);
    }
    printf("\n");
}

void
manipulator::getInverseWandMatrix(pfMatrix &m)
{
    float or[3],pos[3];

    CAVEGetPosition(CAVE_WAND_NAV,pos);
    CAVEGetOrientation(CAVE_WAND_NAV,or);

    m.makeRot(-or[1],0.0,1.0,0.0);
    m.preRot(-or[0],1.0,0.0,0.0,m);
    m.preRot(-or[2],0.0,0.0,1.0,m);
    m.preTrans(-pos[0],-pos[1],-pos[2],m);
}

void
manipulator::getWandMatrix(pfMatrix &m)
{
    float or[3],pos[3];

    CAVEGetPosition(CAVE_WAND_NAV,pos);
    CAVEGetOrientation(CAVE_WAND_NAV,or);

    m.makeTrans(pos[0] ,pos[1] ,pos[2]);
    m.preRot(or[2],0.0,0.0,1.0,m);
    m.preRot(or[0],1.0,0.0,0.0,m);
    m.preRot(or[1],0.0,1.0,0.0,m);
}


void
manipulator::mtxCpy(float m[4][4], pfMatrix *cMat)
{
    for (int r=0; r<4; r++)
        cMat->getRow(r, &m[r][0], &m[r][1], &m[r][2], &m[r][3]);
}

