/**************************************************************************
 **                              myEnums.h                               **
 **************************************************************************
 ** Enumerated types for Widgets used in this application                **
 ** NOTE:  it is likely that not all State and Ident enums defined       **
 **        here are used in this example application.  some Scope        **
 **        classes might be totally ignored by this application.         **
 **************************************************************************/

#ifndef _myEnums_h_
#define _myEnums_h_

#ifdef _STANDARD_C_PLUS_PLUS
    // If -LANG:std is specified, it defines the macro
    // in the preceding line.  Use new-style headers
    // and a using directive to bring names from the
    // std namespace into the global namespace.
	#include<complex>
	#include<iostream>
	using namespace std;
#else
    // If -LANG:std is not specified, use old-style headers,
    // and there is no need for a using directive.
    #include<complex.h>
    #include<iostream.h>
#endif

#include <hash_map>

class mainInterfaceStateScope
{
public:
	enum mainInterfaceState {
		PARENT   = 0,
		MANIP    = 1,
		SCALE    = 2,
		OBSERVER = 3,
		NA       = -1
	};
};

typedef mainInterfaceStateScope::mainInterfaceState mainInterfaceState;

//-----------------------------------------------------------------------

class mainInterfaceTypeScope
{
public:
	enum mainInterfaceType {
		MAIN_MENU = 0,
		NA        = -1
	};
};

typedef mainInterfaceTypeScope::mainInterfaceType mainInterfaceType;

//-----------------------------------------------------------------------

class buttonInterfaceTypeScope
{
public:
	enum buttonInterfaceType {
		MAIN      = 0,
		FANCY     = 1,
		NA        = -1
	};
};

typedef buttonInterfaceTypeScope::buttonInterfaceType buttonInterfaceType;

//-----------------------------------------------------------------------

class buttonIdentTypeScope
{
public:
	enum buttonIdentType {
		MANIP        = 0,
		SCALE        = 1,
		UNKNOWN      = 2,
		FANCY        = 4,
		MAG1         = 7,
		MAG2         = 8,
		RESET        = 9,
		NA           = -1
	};

	// hash_map key equality function
	struct eqstr {
		bool operator()(const char* s1, const char* s2) const
		{
			// returns TRUE if there's a match, else FALSE
			return strcmp(s1, s2) == 0;
		}
	};


	// shorten this down a bit for here and cncreteMed
	typedef
	hash_map<const char *, buttonIdentType, hash<const char *>, eqstr> myMap;

	static myMap btnMap;

	// fills our hashed associative array for ID'ing buttons
	// called from cncreteMed constructor.
	static void fillBtnMap() {
		btnMap ["MANIP"] = buttonIdentTypeScope::MANIP;
		btnMap ["SCALE"] = buttonIdentTypeScope::SCALE;
		btnMap ["FANCY"] = buttonIdentTypeScope::FANCY;
		btnMap ["UNKNOWN"] = buttonIdentTypeScope::UNKNOWN;
		btnMap ["MAG1"] = buttonIdentTypeScope::MAG1;
		btnMap ["MAG2"] = buttonIdentTypeScope::MAG2;
		btnMap ["RESET"] = buttonIdentTypeScope::RESET;
		btnMap ["NA"] = buttonIdentTypeScope::NA;
	}
};

typedef buttonIdentTypeScope::buttonIdentType buttonIdentType;

//-----------------------------------------------------------------------

class sliderInterfaceTypeScope
{
public:
	enum sliderInterfaceType {
		FANCY       = 0,
		NA          = -1
	};
};

typedef sliderInterfaceTypeScope::sliderInterfaceType sliderInterfaceType;

//-----------------------------------------------------------------------

class sliderIdentTypeScope
{
public:
	enum sliderIdentType {
		SLIDERC1    = 6,
		SLIDERC2    = 7,
		SLIDERL1    = 8,
		SLIDERH1    = 9,
		SLIDERL2    = 10,
		SLIDERH2    = 11,
		NA          = -1
	};
};

typedef sliderIdentTypeScope::sliderIdentType sliderIdentType;

//-----------------------------------------------------------------------

class incrementorInterfaceTypeScope
{
public:
	enum incrementorInterfaceType {
		FANCY  = 0,
		NA     = -1
	};
};

typedef incrementorInterfaceTypeScope::incrementorInterfaceType
                                       incrementorInterfaceType;

//-----------------------------------------------------------------------

class incrementorIdentTypeScope
{
public:
	enum incrementorIdentType {
		CENTER1   = 0,
		CENTER2   = 1,
		WIDTH1    = 2,
		WIDTH2    = 3,
		NA       = -1
	};
};

typedef incrementorIdentTypeScope::incrementorIdentType incrementorIdentType;

//-----------------------------------------------------------------------

#endif // #ifndef _myEnums_h_

